#!/bin/ksh
# Common functions for Sun Internet Mail Server email user interface.

# Copyright (c) 1997 Sun Microsystems, Inc.
# @(#)dirupdate.cgi	1.3 97/07/24

PATH=/usr/bin

# We only refer to a couple binaries outside of /usr/bin, use
# explict paths for them, and find them via pkgparam.

LDAPBASEDIR=$(pkgparam SUNWsds BASEDIR 2>/dev/null || print /opt)/SUNWconn/ldap/bin
HTMLBASE=$(pkgparam SUNWimadm BASEDIR 2>/dev/null || print /)/opt/SUNWmail/html
LIBBASE=$(pkgparam SUNWimadm BASEDIR 2>/dev/null || print /)/opt/SUNWmail/lib

function isAllWhiteSpace {
	return $#
}

# Get the fully qualified DNS domain name for this host.  Since the
# domainname(1) command returns the NIS/NIS+ domainname, which is
# not necessarily the DNS domain name, first try to pick the domain
# entry out of /etc/resolv.conf.  If /etc/resolv.conf is not
# present, use domainname(1) as the best available guess.
# Set global variable DOMAINNAME to the domain.
#

function getdomainname {
	
	DOMAINNAME=""
	if [ -f /etc/resolv.conf ]
	then
		DOMAINNAME=$(/usr/bin/grep '^domain' /etc/resolv.conf | /usr/bin/cut -d ' ' -f 2)
	fi
	if [ "$DOMAINNAME" = "" ]
	then
		DOMAINNAME=$(/usr/bin/domainname)
	fi
	return 0
}

# getsearchbase
#
# Search the LDAP domain subtree for the proper DN of the DIT for the
# server.  Set LDAPSEARCHBASE to this DN.

function getsearchbase {
	typeset names
	typeset OLDIFS="$IFS"
	IFS=.
	set -A names $DOMAINNAME
	IFS="$OLDIFS"
	typeset -i levelmax=${#names[*]}
	typeset basename="dc=${names[levelmax-2]},dc=${names[levelmax-1]}"
	typeset -i levelcur=0
	typeset thisname
	
	while ((levelcur+2 <= levelmax))
	do
		typeset -i n
		thisname="$basename"
		let "n = levelmax - 3"
		while ((n >= levelcur))
		do
			thisname="dc=${names[n]},$thisname"
			let "n = n-1"
		done
		
		LDAPSEARCHBASE=$(${LDAPBASEDIR}/ldapsearch -h $LDAPHOST -b "$thisname" "objectclass=domain" associatedname 2>/dev/null | /usr/bin/sed -e '1d' -e '3,$d' -e 's/^[^=]*=//')
		if [ "$LDAPSEARCHBASE" ]
		then
			return 0;
		fi
		let "levelcur = levelcur+1"
	done
	return 1
}



# finddn uid
# Finds the distinguished name for the given uid. Parameter DN is set
# to the result.
# Exit codes:
# 0 - OK, DN set as requested
# 1 - Error, search failed (almost certainly means uid not found)
# 2 - Error, multiple entries match (unless uid has a * this is an
#     administrative error, two entries have the same uid.)

function finddn {
	typeset uid="$1"
	typeset line
	typeset -i lcount=0

	${LDAPBASEDIR}/ldapsearch -h $LDAPHOST -b "$LDAPSEARCHBASE" "uid=$uid" dn  2>&1 |&
	while read -p line
	do
		DN="$line"
		lcount=lcount+1
	done

	if (( lcount < 1 ))
	then
		return 1
	elif (( lcount > 1 ))
	then
		return 2
	else
		return 0
	fi
}

#
# modify "replace"|"add"|"delete" attrname value
# Helper function to replace LDAP attributes.
# Expects parameters LDAPHOST, DN, and oldpassword to be set correctly.
# Exit code is that of ldapmodify: 0 = OK, nonzero = notOK
# replace or delete with a null value is treated as a delete of the entire
# attribute.

function modify {
	typeset op="$1" attrname="$2" value="$3"

	if [ "$value" ]
	then
		

		${LDAPBASEDIR}/ldapmodify -h $LDAPHOST \
		 -D "$DN" -w "$oldpassword" \
		 <<EOF 1>/dev/null 2>&1
dn: $DN
changetype: modify
${op}: $attrname
${attrname}: $value
EOF
		return $?
	fi
	
	case $op in
		@(replace|delete))
		    ${LDAPBASEDIR}/ldapmodify -h $LDAPHOST \
		     -D "$DN"  -w "$oldpassword" \
		     <<EOF 1>/dev/null 2>&1
dn: $DN
changetype: modify
delete: $attrname
EOF
# delete returns nonzero (actually 16, but not in man page) if attribute
# isn't there.  We don't want to treat that as an error.  So pretend
# delete always works, sigh.
		    return 0
		    ;;
		*)
		    return 1
		    ;;
	esac
}


#
# getattr attrname [value]
# Helper function to retrieve LDAP attributes.
# Omit value for single-valued attributes. For multi-value attributes,
# provide the value and the output will be either the value or null.
# Expects parameters LDAPHOST, and DN to be set correctly.
# Warning: Dealing with multiple values is problematic if value might
# contain spaces.
# Exit code is that of ldapsearch: 0 = OK, nonzero = notOK

function getattr {
	typeset attrname="$1" value="$2"
	typeset line

	if [ "$value" ]
	then
		${LDAPBASEDIR}/ldapsearch -h $LDAPHOST -b "$DN" "$attrname=$value" $attrname | /usr/bin/sed -e '1d' -e 's/^[^=]*=//'  | /usr/bin/grep -i "$value"
	else
		${LDAPBASEDIR}/ldapsearch -h $LDAPHOST -b "$DN"  "$attrname=*" $attrname | /usr/bin/sed -e '1d' -e 's/^[^=]*=//'
	fi
	
}
# setpassword uid oldpassword newpassword
# Changes the password of uid from oldpassword to newpassword
# Exit codes:
# 0 - OK, password changed.
# 1 - Error, uid not found
# 2 - Error, multiple matching uids in directory (see finddn)
# 3 - Error, update failed (probably old password incorrect.)

function setpassword {
	typeset uid="$1" oldpassword="$2" newpassword="$3"
	typeset -i rc

	finddn $uid
	rc=$?

	if ((rc != 0))
	then
		return $rc
	fi

	if modify replace userpassword "$newpassword"
	then
		return 0
	else
		return 3
	fi
}

# setvacation uid oldpassword onoff expdate subject text itext
# Sets the vacation parameters for a user.
# uid = uid
# oldpassword = current password
# onoff = "on" or "off" to enable or disable vacation replies
# expdate = day to stop vacation replies, already in UTC (YYMMDDHHMMSSZ) fmt
# subject = autoreply subject string
# text = autoreply body text, already encoding newline as $
# itext = autoreply internal body text, already encoding newline as $
# Exit codes:
# 0 - OK, vacation updates made.
# 1 - Error, uid not found
# 2 - Error, multiple matching uids in directory (see finddn)
# 3 - Error, update failed (probably old password incorrect.)

function setvacation {
	typeset uid="$1" oldpassword="$2" onoff="$3" expdate="$4"
	typeset subject="$5" text="$6" itext="$7"
	typeset -i rc

	finddn $uid
	rc=$?
	
	if ((rc != 0))
	then
		return $rc
	fi

	case "$onoff" in
		# It's a shame, but we don't check return codes from
		# ldapmodify here because turning vacation on if it is already
		# on or off if it is already off generates an error and
		# we can't distinguish that error from the others
		# as the only defined return codes are 0 and "non-zero", see
		# ldapmodify(1).  If the password is wrong we'll catch it
		# when we update the other attributes.
		on)
			modify add mailAutoReplyMode vacation
			modify add mailDeliveryOption autoreply
			;;
		off)
			modify delete mailAutoReplyMode vacation
			modify delete mailDeliveryOption autoreply
			;;
	esac

	# Fortunately, replacing an attribute that does not exist
	# does an add. 

	if isAllWhiteSpace $text
	then
		text=""
	fi
	
	if isAllWhiteSpace $itext
	then
		itext=""
	fi

	if modify replace mailAutoReplyExpirationDate "$expdate" && \
	 modify replace mailAutoReplySubject "$subject" && \
	 modify replace mailAutoReplyText "$text" && \
	 modify replace mailAutoReplyTextInternal "$itext"
	then
		return 0
	else
		return 3
	fi
}


# Set up global variables.  Pick these up from
# /etc/opt/SUNWmail/admin/userupdate, or attempt to deduce from 
# system configuration.
# LDAPHOST=name of LDAP server host.  Usually same host we are on.
# LDAPSEARCHBASE=Distinguished Name of base of Directory Info Tree.
# DOMAINNAME=DNS domain to use to deduce LDAPSEARCHBASE.


defaultsfile=/etc/opt/SUNWmail/admin/userupdate

if [[ -f $defaultsfile ]]
then
	. $defaultsfile
fi

if [ "$LDAPHOST" ]
then
	# LDAPHOST set by defaults, use it as specified.
	:
elif [ "$SERVER_NAME" ]
then
	# We are a CGI script and have been told our hostname	
	LDAPHOST=$SERVER_NAME
else
	# Assume execution host
	LDAPHOST=$(uname -n)
fi

if [ "$DOMAINNAME" ]
then
	# DOMAINNAME set by defaults, use it as specified.
	:
else
	# Deduce from system configuration
	getdomainname
fi

if [ "$LDAPSEARCHBASE" ]
then
	# LDAPSEARCHBASE set by defaults, use it as specified.
	:
else
	# Deduce from domain name
	getsearchbase
fi

# setprogram uid oldpassword program
# Sets the delivery program for a user.
# uid = uid
# oldpassword = current password
# oldprgm = current program
# Exit codes:
# 0 - OK, program updates made.
# 1 - Error, uid not found, then search for cn as it might be a list.
# 2 - Error, multiple matching uids in directory (see finddn)
# 3 - Error, update failed (probably old password incorrect.)
 
function setprogram {
        typeset uid="$1" oldpassword="$2" oldprgm="$3" 
        typeset -i rc
 
        finddn $uid
        rc=$?
        
	if ((rc == 1))
        then
            findlistdn $uid
            rc=$?
        fi

        if ((rc != 0))
        then
                return $rc
        fi
 
        case "$oldprgm" in 
                None) 
                        modify delete mailDeliveryOption program
			modify delete mailProgramDeliveryInfo
                        ;; 
                *) 
                        modify add mailDeliveryOption program 
                        ;; 
        esac 
 
	if [ "$oldprgm" != "None" ]
	then
        	if modify replace mailProgramDeliveryInfo "$oldprgm" 
        	then
               	 return 0
        	else
               	 return 3
        	fi
	fi
}

# checkpasswd uid dn password
# Checks the password of uid, by trying to do an ldapbind.
# Exit codes:
# 0 - OK, right password.
# 1 - Error, incorrect password.
function checkpasswd {
        typeset uid="$1"
	typeset dn="$2"
        typeset passwd="$3"

        ${LDAPBASEDIR}/ldapsearch -h $LDAPHOST -b "$LDAPSEARCHBASE" -D "$dn" -w "$passwd" "uid=$uid"  1>/dev/null 2>&1
	rc=$?
	return $rc
}

# setlocal uid password
# Sets the delivery program for a user to local.
# uid = uid
# password = current password
# Exit codes:
# 0 - OK, program updates made.
# 1 - Error, uid not found
# 2 - Error, multiple matching uids in directory (see finddn)
# 3 - Error, update failed (probably password incorrect.)
function setlocal {
        typeset uid="$1" oldpassword="$2"  
        typeset -i rc
 
        finddn $uid
        rc=$?
        
        if ((rc != 0))
        then
                return $rc
        fi

	modify delete mailDeliveryOption forward
	modify delete mailForwardingAddress

	foldertype=$(getattr mailFolderMap SUN-MS)
        typeset -l lfoldertype=$foldertype
        if [ $lfoldertype = sun-ms ]
        then
	  modify add mailDeliveryOption mailbox
        else
	  modify add mailDeliveryOption native
	fi
	
	return 0
}

# setforward uid password forward_addresses
# Sets the delivery program for a user to local.
# uid = uid
# password = current password
# forward_addresses = addresses to forward to
# Exit codes:
# 0 - OK, program updates made.
# 1 - Error, uid not found
# 2 - Error, multiple matching uids in directory (see finddn)
# 3 - Error, update failed (probably password incorrect.)
function setforward {
        typeset uid="$1" oldpassword="$2" forward="$3" 
        typeset -i rc
 
        finddn $uid
        rc=$?
        
        if ((rc != 0))
        then
                return $rc
        fi

	modify delete mailDeliveryOption mailbox
	modify delete mailDeliveryOption native	
	modify delete mailForwardingAddress

	modify add mailDeliveryOption forward
	for i in ${forward}
	do
	  modify add mailForwardingAddress "$i"
	  rc=$?
	  if ((rc != 0))
          then
                return $rc
          fi
	done
	return 0
}

# setforwardlocal uid password forward_addresses
# Sets the delivery program for a user to local.
# uid = uid
# password = current password
# forward_addresses = addresses to forward to
# Exit codes:
# 0 - OK, program updates made.
# 1 - Error, uid not found
# 2 - Error, multiple matching uids in directory (see finddn)
# 3 - Error, update failed (probably password incorrect.)
function setforwardlocal {
        typeset uid="$1" oldpassword="$2" forward="$3" 
        typeset -i rc
 
        finddn $uid
        rc=$?
        
        if ((rc != 0))
        then
                return $rc
        fi
	
	modify delete mailForwardingAddress
	modify add mailDeliveryOption forward
	for i in ${forward}
	do
	  modify add mailForwardingAddress "$i"
	  rc=$?
	  if ((rc != 0))
          then
                return $rc
          fi
	done

	foldertype=$(getattr mailFolderMap SUN-MS)
        typeset -l lfoldertype=$foldertype
        if [ $lfoldertype = sun-ms ]
        then
	  modify add mailDeliveryOption mailbox
        else
	  modify add mailDeliveryOption native
	fi

	return 0
}

# findlistdn cn
# Finds the distinguished name for the given cn. Parameter DN is set
# to the result.
# Exit codes:
# 0 - OK, DN set as requested
# 1 - Error, search failed (almost certainly means cn not found)
# 2 - Error, multiple entries match (unless cn has a * this is an
#     administrative error, two entries have the same cn.)
 
function findlistdn {
        typeset cn="$1"
        typeset line
        typeset -i lcount=0
 
        ${LDAPBASEDIR}/ldapsearch -h $LDAPHOST -b "$LDAPSEARCHBASE" "cn=$cn" dn  2>&1 |&       
        while read -p line
        do
                DN="$line"
                lcount=lcount+1
        done
 
        if (( lcount < 1 ))
        then
                return 1
        elif (( lcount > 1 ))
        then
                return 2
        else
                return 0
        fi
}

